//========================================================================================
//
//	File:				FWClnInf.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWCLNINF_H
#define FWCLNINF_H

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

// ----- ODF Foundation Includes -----

#ifndef FWSTDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWCLAINF_H
#include "FWClaInf.h"
#endif

#ifndef FWITERS_H
#include "FWIters.h"
#endif

#ifndef FWTCOLL_H
#include "FWTColl.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class ODDraft;
class ODShape;

class FW_CFrame;
class FW_CDataInterchange;

//========================================================================================
//	class FW_CCloneInfo
//========================================================================================

class FW_CCloneInfo
{
//----------------------------------------------------------------------------------------
//	Constructors/Destructor
//
public:
	FW_DECLARE_AUTO(FW_CCloneInfo)

	FW_CCloneInfo(Environment* ev,
				ODDraft* fromDraft, 
				FW_CFrame* scopeFrame, 
				ODCloneKind cloneKind);
	FW_CCloneInfo(Environment* ev,
				ODDraftKey key,
				ODDraft* fromDraft, 
				FW_CFrame* scopeFrame, 
				ODCloneKind cloneKind);
	~FW_CCloneInfo();

//----------------------------------------------------------------------------------------
//	API
//
public:
	void					BeginClone(Environment* ev, ODDraft* destinationDraft);
	void					EndClone(Environment* ev);
	void					AbortClone(Environment* ev);
	
	ODID					Clone(Environment* ev,
								ODID fromObjectID,
								ODID toObjectID,
								ODID scope);
	
	FW_CFrame*				GetScopeFrame(Environment* ev) const;
	ODCloneKind				GetCloneKind(Environment* ev) const;
	ODDraft*				GetFromDraft(Environment* ev) const;
	ODDraft*				GetToDraft(Environment* ev) const;
	FW_CDataInterchange*	GetDataInterchange(Environment* ev) const;
	ODDraftKey				GetDraftKey(Environment* ev) const;
	
//----------------------------------------------------------------------------------------
//	Data Members
//
public:
	FW_TOrderedCollection<FW_MProxy>*	fClonedProxyList;			// out	Cloning content
	ODID								fClonedPartID;				// out	Cloning part/Cloning Single Embedded Frame
	ODID								fClonedFrameID;				// out	Cloning Single Embedded Frame
	ODShape*							fFrameShape;				// out 	Cloning part/Cloning Single Embedded Frame

private:
	ODDraftKey 							fKey; 						// in
	ODDraft* 							fFromDraft; 				// in
	FW_CFrame* 							fScopeFrame;				// in
	ODCloneKind							fCloneKind;					// in
	FW_Boolean							fCloning;
	FW_Boolean							fBeganClone;
	FW_CDataInterchange*				fDataInterchange;
	ODDraft* 							fToDraft; 					// in
};

//========================================================================================
//	Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CCloneInfo::GetScopeFrame
//----------------------------------------------------------------------------------------
inline FW_CFrame* FW_CCloneInfo::GetScopeFrame(Environment*) const
{
	return fScopeFrame;
}

//----------------------------------------------------------------------------------------
//	FW_CCloneInfo::GetCloneKind
//----------------------------------------------------------------------------------------
inline ODCloneKind FW_CCloneInfo::GetCloneKind(Environment*) const
{
	return fCloneKind;
}

//----------------------------------------------------------------------------------------
//	FW_CCloneInfo::GetFromDraft
//----------------------------------------------------------------------------------------
inline ODDraft* FW_CCloneInfo::GetFromDraft(Environment*) const
{
	return fFromDraft;
}

//----------------------------------------------------------------------------------------
//	FW_CCloneInfo::GetToDraft
//----------------------------------------------------------------------------------------
inline ODDraft* FW_CCloneInfo::GetToDraft(Environment*) const
{
	return fToDraft;
}

#endif

