//========================================================================================
//
//	File:				FWFrame.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWFRAME_H
#define FWFRAME_H

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWPRTDEF_H
#include "FWPrtDef.h"
#endif

#ifndef FWSVIEW_H
#include "FWSView.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWDRGDRP_H
#include "FWDrgDrp.h"
#endif

#ifndef FWSCROLR_H
#include "FWScrolr.h"
#endif

// ----- OS Includes -----

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWFCTINF_H
#include "FWFctInf.h"
#endif

#ifndef FWACQUIR_H
#include "FWAcquir.h"
#endif

#ifndef FWICON_H
#include "FWIcon.h"
#endif

// ----- Foundation Includes -----

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWSTRING_H
#include "FWString.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODFrame_xh
#include <Frame.xh>
#endif

#ifndef FWODTYPS_H
#include "FWODTyps.h"
#endif

#ifndef SOM_ODStorageUnit_xh
#include <StorageU.xh>
#endif

//========================================================================================
//	Forward Declaration
//========================================================================================

class ODFacet;
class ODWindow;
class ODShape;
class ODFocusSet;
class ODStorageUnitView;

class FW_CPart;
class FW_CMenuBar;
class FW_CMenuEvent;
class FW_CWindow;
class FW_CGraphicDevice;
class FW_CMouseEvent;
class FW_CActivateEvent;
class FW_CClipboardCommand;
class FW_CPresentation;
class FW_CResumeSuspendEvent;
class FW_CCloneInfo;
class FW_CViewAs;
class FW_CInterest;
class FW_CPrintHandler;
class FW_CScrollingViewIterator;
class FW_CIdler;
class FW_CScroller;
class FW_CPrivBaseScroller;
class FW_MDraggableFrame;
class FW_MDroppableFrame;
class FW_CButton;
class FW_CSelection;

//========================================================================================
//	class FW_CFrame
//========================================================================================

class FW_CFrame : public FW_CSuperView
{
public:
	FW_DECLARE_CLASS
	FW_DECLARE_AUTO(FW_CFrame)

	friend class FW_CPart;
	friend class FW_CIdler;

//----------------------------------------------------------------------------------------
//	Constructors/Destructors
//
public:
	FW_CFrame(Environment*			ev,
				ODFrame* 			frame, 
				FW_CPresentation* 	presentation,
				FW_CPart* 			part);			
	virtual ~ FW_CFrame();

//----------------------------------------------------------------------------------------
//	API from ODFrame (some are just inlines)
//
public:
	ODID						GetFrameGroup(Environment* ev) const;
	ODID						GetSequenceNumber(Environment* ev) const;

	FW_Boolean					IsPersistent(Environment* ev) const;
						
	FW_Boolean					IsRoot(Environment* ev) const;

	FW_Boolean					IsSubframe(Environment* ev) const;
	void						SetSubframe(Environment* ev, 
									FW_Boolean isSubframe);

	FW_Boolean					IsOverlaid(Environment* ev) const;

	FW_Boolean					IsInLimbo(Environment* ev) const;
	
	FW_Boolean					IsFrozen(Environment* ev) const;
	void						SetFrozen(Environment* ev, 
									FW_Boolean isFrozen);

	FW_Boolean					DoesPropagateEvents(Environment* ev) const;
	void						SetPropagateEvents(Environment* ev, 
									FW_Boolean doesPropagateEvents);

	FW_CPresentation*			GetPresentation(Environment* ev) const;
	ODTypeToken					GetViewType(Environment* ev) const;

	void						ChangeUsedShape(Environment* ev, 
									ODShape* newUsedShape, 
									ODCanvas* biasCanvas = NULL);
	FW_Boolean					RequestFrameShape(Environment* ev, 
									ODShape* shape,
									ODCanvas* biasCanvas = NULL);

	ODShape*					AcquireUsedShape(Environment* ev, 
									ODCanvas* biasCanvas = NULL) const;	
	ODTransform*				AcquireInternalTransform(Environment* ev, 
									ODCanvas* biasCanvas = NULL) const;	
	ODShape*					AcquireFrameShape(Environment* ev, 
									ODCanvas* biasCanvas = NULL) const;

	ODFrame*					AcquireContainingFrame(Environment* ev) const;
	
	ODWindow*					AcquireODWindow(Environment* ev) const;

	ODID						GetID(Environment* ev) const;
	
//----------------------------------------------------------------------------------------
//	Inherited from FW_MEventHandler
//
public:
	// ----- Events -----
	virtual FW_MEventHandler*	GetTarget(Environment* ev) const;
	virtual void				SetTarget(Environment* ev, FW_MEventHandler* newTarget, FW_Boolean tabSelection = FALSE);
	virtual FW_Boolean			WantsToBeTarget(Environment* ev);
	virtual FW_Handled			HandleIdle(Environment* ev, const FW_CNullEvent& theNullEvent);

	// ----- Menus -----
	virtual FW_Handled			HandleAdjustMenus(Environment* ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);
	virtual FW_Handled			HandleMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);

//----------------------------------------------------------------------------------------
//	Inherited from FW_CSuperView
//
public:
	virtual void				SetLocation(Environment* ev, const FW_CPoint& location, FW_ERedrawVerb redraw = FW_kInvalidate);
	virtual void				HandleActivateEvent(Environment* ev, const FW_CActivateEvent& theActivateEvent);	

	virtual void				SubViewWasRemoved(Environment* ev, FW_CView* subView);
	
	virtual FW_CFrame*			GetFrame(Environment* ev) const;
	virtual FW_Boolean			IsFrame(Environment* ev) const;
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	// ----- Drag and Drop -----
	FW_MDraggableFrame*			GetDraggable(Environment* ev) const;
	FW_MDroppableFrame*			GetDroppable(Environment* ev) const;

	FW_Boolean					IsTopFrame(Environment* ev) const;
	
	static FW_CFrame* 			ODtoFWFrame(Environment* ev, ODFrame* odFrame);
	ODFrame*					GetODFrame(Environment *ev) const;

	FW_Boolean					IsSiblingFrameOf(Environment *ev, ODFrame* otherODFrame) const;
	FW_Boolean					HasSamePartEditorAs(Environment *ev, ODFrame* otherODFrame) const;
	FW_Boolean					HasSameDraftAs(Environment* ev, ODFrame* otherODFrame) const;

	// ----- Mouse Tracking -----
	virtual FW_Handled			AdjustCursor(Environment* ev, ODFacet* odFacet, const FW_CPoint& theMousePoint, ODEventInfo* eventInfo);
	
	virtual FW_Handled			HandleMouseEnter(Environment* ev, ODFacet* odFacet, const FW_CPoint& where, ODEventInfo* eventInfo);
	virtual FW_Handled			HandleMouseLeave(Environment* ev, ODFacet* odFacet, ODEventInfo* eventInfo);
	virtual FW_Handled			HandleMouseWithin(Environment* ev, ODFacet* odFacet, const FW_CPoint& theMousePoint, ODEventInfo* eventInfo);
		
	// ----- Events ------
	virtual FW_Handled			HandleSuspendResumeEvent(Environment* ev, const FW_CSuspendResumeEvent& theResumeSuspendEvent);
#ifdef FW_BUILD_MAC
	virtual FW_Handled			HandleWindowEvent(Environment* ev, const FW_CMacWindowEvent& windowEvent);
#endif
	
	virtual FW_Handled			HandleVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent);

	// ----- Facet Management -----		
	virtual void 				FacetAdded(Environment* ev, ODFacet* odFacet, unsigned short facetCount);
	virtual void 				FacetRemoved(Environment* ev, ODFacet* odFacet, unsigned short facetCount);

	virtual void				CanvasChanged(Environment* ev, ODFacet* facet);

	unsigned short				GetFacetCount(Environment* ev) const;
	
	// ----- Containing Part Properties -----
	FW_Boolean					GetAdoptsContainerProperties(Environment* ev) const;
	void						SetAdoptsContainerProperties(Environment* ev, FW_Boolean adoptsProperties);
	
	virtual void				SynchContainingPartProperties(Environment* ev);
	virtual void				AdoptContainingPartProperties(Environment* ev, ODStorageUnit* propertyUnit);

	// ----- Add/Remove frame notification -----
	virtual void				FrameAdded(Environment* ev, ODFrame* odFrame, FW_Boolean fromStorage);
	virtual void				FrameRemoved(Environment* ev, FW_Boolean toStorage);
		
	// ----- Focus Management -----
	virtual void 				FocusStateChanged(Environment* ev, ODTypeToken focus, FW_Boolean newState, ODFrame* newOwner);	
	virtual FW_Boolean			BeginRelinquishFocus(Environment *ev, ODTypeToken focus, ODFrame* proposedFrame);
	virtual void				AbortRelinquishFocus(Environment *ev, ODTypeToken focus, ODFrame* proposedFrame);
	virtual ODFocusSet*			GetFocusSet(Environment* ev) const;	
	
	virtual	void				RelinquishFocusSet(Environment *ev);
	virtual FW_Boolean			RequestFocusSet(Environment *ev);
		
	// ----- Window Geometry -----
	virtual void				AdjustZoomedWindowSize(Environment *ev, FW_CPoint& proposedSize);
	virtual void				AdjustWindowGrowLimits(Environment *ev, FW_CPoint& minSize, FW_CPoint& maxSize);

	// ----- Menus -----
	virtual void				InstallMenus(Environment* ev);
	
	// ----- ViewType/Presentation/Sequence -----
	virtual void 				ViewTypeChanged(Environment* ev, ODTypeToken newViewType, ODTypeToken oldViewType);
	virtual void 				SequenceChanged(Environment* ev);
	
	void						ChangePresentation(Environment* ev, FW_CPresentation* presentation);
	virtual void 				PresentationChanged(Environment* ev);
	
	// ----- Frame Geometry -----
	virtual void 				FrameShapeChanged(Environment* ev);
	virtual void				GeometryChanged(Environment *ev,
											ODFacet* odFacet,
											FW_Boolean clipShapeChanged,
											FW_Boolean transformChanged);
	
	// ----- Requested Frame -----
	FW_Boolean					IsRequestedFrame(Environment* ev) const;
								 
	// ----- Used and Update Shape -----
	void 						UpdateUsedAndActiveShapes(Environment* ev);
	void						UpdateUsedShape(Environment* ev);
	void						UpdateActiveShape(Environment* ev, ODFacet* facet);

	virtual ODShape* 			AdjustUsedShape(Environment* ev, ODShape* suggestedUsedShape);
	virtual ODShape* 			AdjustActiveShape(Environment* ev, ODFacet* facet, ODShape* suggestedActiveShape);
	
	// ----- Window -----
	FW_CWindow*					GetWindow(Environment* ev) const;
	virtual FW_CWindow* 		NewPartWindow(Environment* ev, 
											const FW_CRect& suggestedWindowRect, 
											const FW_CString& suggestedWindowTitle);
	
	// ----- Focus -----
	FW_Boolean					HasSelectionFocus(Environment* ev) const;
	FW_Boolean					HasModalFocus(Environment* ev) const;
	
	// ----- Frame Activation -----
	void 						ActivateFrame(Environment* ev, ODFacet* activatedFacet);
	FW_Boolean					ActivateWindow(Environment* ev, ODFacet* facet, FW_Boolean activateFrame);

	// ----- Getter/Setter -----
	FW_CPart* 					GetPart(Environment* ev) const;
	ODPart* 					GetODPart(Environment* ev) const;

	FW_CSuperView*				GetContentView(Environment* ev) const;
	void 						SetContentView(Environment* ev, FW_CSuperView* view);
	
	FW_Boolean					GetFrontClicks(Environment* ev) const;
	void						SetFrontClicks(Environment* ev, FW_Boolean getFrontClicks);

	ODFacet* 					GetActiveFacet(Environment* ev) const;
	virtual void				SetActiveFacet(Environment* ev, ODFacet* activeFacet);
		
	virtual FW_Boolean			CanBeActiveFrame(Environment* ev) const;
	void						SetCanBeActiveFrame(Environment* ev, FW_Boolean beActive);
	
	FW_CFrame*					GetSourceFrame(Environment* ev);

	// ----- Cloning -----
	virtual void				ClonePartInfo(Environment* ev, 
									ODDraftKey key,
									ODInfoType partInfo,
									ODStorageUnitView* storageUnitView,
									ODFrame* scope);

	// ----- Scrolling -----
	FW_CScroller*				GetScroller(Environment* ev) const;
	void						AdoptScroller(Environment* ev, FW_CScroller* scroller);
 
  	FW_Boolean					Reveal(Environment* ev, ODShape* shapeToReveal = NULL);
 	
	// ----- Clipboard -----
	FW_Boolean					HasPropertyOnClipboard(Environment* ev, 
									ODPropertyName propertyName, 
									ODValueType type) const;
	FW_Boolean					HasSupportedKindOnClipboard(Environment* ev) const;

	// ----- Linking -----
	FW_Boolean					IsFrameInLinkDestination(Environment* ev);
		
	// ----- Rendering -----
	virtual FW_Handled			HandleDraw(Environment* ev, ODFacet* odFacet, ODShape* invalidShape);	
	virtual void 				Draw(Environment* ev, ODFacet* odFacet, ODShape* invalidShape);
	
	// ----- Highlight -----
	virtual void				HighlightChanged(Environment* ev, ODFacet* odFacet, ODHighlight from, ODHighlight to);
	virtual void				DrawFullHighlight(Environment *ev, ODFacet* facet, ODShape* invalidShape);
	
	// ----- View As -----
	virtual void				BuildThumbnail(Environment* ev, FW_CGraphicContext& gc, FW_CRect& usedRect);
	virtual FW_CIcon			GetIcon(Environment* ev, short iconSize) const;
	virtual FW_CViewAs*			NewViewAs(Environment* ev, ODTypeToken viewAs);
	
	// ----- Content -----
	void						GetContentExtent(Environment* ev, FW_CPoint& extent) const;
	ODShape* 					AcquireContentShape(Environment *ev);
	
	// ----- Commands/Undo -----
	virtual FW_CClipboardCommand*	NewClipboardCommand(Environment* ev, ODCommandID commandID);
		// Override to return a subclass of FW_CClipboardCommand
	
	virtual FW_Handled			ShowPartInfo(Environment* ev);

	// ----- Printing ------
	virtual FW_CPrintHandler*	NewPrintHandler(Environment* ev);

	// ----- Persistence -----
	virtual void				InternalizeFrame(Environment* ev, ODStorageUnitView* storageUnitView);
	virtual void				ExternalizeFrame(Environment* ev, ODStorageUnitView* storageUnitView);

	// ----- View Editing -----
	virtual void				DeleteAndResetViews(Environment* ev);
	
private:
#ifdef FW_BUILD_WIN
	void						CreateShadowWindow(Environment* ev);
	static LRESULT CALLBACK 	ShadowWindowProc(HWND hWnd, UINT msg,
											 WPARAM wParam, LPARAM lParam);
	void						MoveSizeShadowWindow(Environment* ev);
	
public:
	ODPlatformWindow			GetShadowWindow(Environment* ev) const;
#endif

//----------------------------------------------------------------------------------------
//	Internal API
//
protected:
	virtual void				PrivCreateSubViewsFromStream(Environment* ev, FW_CReadableStream& stream);
	void						PrivInstantiateViews(Environment* ev);
	
	void						PrivGetFrameShapeBounds(Environment* ev, FW_CRect& rect) const;
		
#ifdef FW_BUILD_MAC
 	void 						PrivMacInGrowBox(Environment* ev, const FW_CMacWindowEvent& windowEvent);
#endif

public:	
	virtual void				PrivStartStopDragging(Environment *ev, FW_EPrivDraggingFlags dragFlags);

	virtual void				PrivAttachSourceFrame(Environment* ev, FW_CFrame* sourceFrame);

	FW_CWindow*					PrivCreateWindow(Environment* ev, ODWindow* odWindow);

	ODTypeToken					PrivGetPreviousViewType(Environment *ev) const;
	void						PrivSetPreviousViewType(Environment *ev, ODTypeToken viewType);

	void						PrivCreateViewAs(Environment* ev);

	void						PrivContentViewLocationChanged(Environment* ev, const FW_CPoint& newLocation);
	void						PrivRealignInternalTransform(Environment* ev);
	
	FW_Boolean					PrivInSharedWindow(Environment *ev) const;
	
	void						PrivFacetAdded(Environment *ev, ODFacet* facet);
	void						PrivFacetRemoved(Environment *ev, ODFacet* facet);
	
	void						PrivSetDraggable(Environment *ev, FW_MDraggableFrame* draggable);
	void						PrivSetDroppable(Environment *ev, FW_MDroppableFrame* droppable, FW_Boolean isDroppable);
	
	void						PrivUpdateScrollParameters(Environment *ev);
	FW_CPoint					PrivGetContentViewOffset(Environment* ev) const;

	void						PrivSetRequestedFrame(Environment *ev);
	
	FW_DroppableState			PrivGetDroppableState() const;
	void						PrivSetDroppableState(FW_DroppableState droppableState);
	
	FW_Boolean 					PrivSubViewsCreated(Environment*);
	
	virtual void				PrivButtonAttached(Environment *ev, FW_CButton* button);

	void						PrivSetInfoCommandAdjusted(Environment*, FW_Boolean value);

	virtual void				PrivContentShapeChanged(Environment *ev);

	void						PrivChangeInternalTransform(Environment* ev, 
									ODTransform* internalTransform,
									ODCanvas* biasCanvas = NULL);
	
	void						PrivRequestScrollingFocus(Environment *ev);

	void						PrivExternalizeFrame(Environment* ev, ODStorageUnitView* storageUnitView);
	void						PrivInternalizeFrame(Environment* ev, ODStorageUnitView* storageUnitView);

	FW_Boolean					PrivGetInfoString(Environment* ev, FW_CSelection* selection, FW_CString& infoString);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	ODStorageUnitID 			fFrameID;				// ID for this display frame
	ODFrame*					fODFrame;				// OpenDoc frame
	FW_CPart*					fPart;

	FW_MEventHandler*			fTarget;				// For key/menu/idle events, never NULL
  	FW_CSuperView*				fContentView;  			// View displaying the content (or frame itself)
	FW_Boolean					fGetFrontClicks;
	FW_Boolean					fNeedsFoci;				// To remember active frame during activate/deactivate events
	FW_Boolean					fNeedsScrollingFocus;	// To remember scrolling focus frame during activate/deactivate events
	FW_Boolean					fHasSelectionFocus;		// TRUE when frame has the selection focus
	FW_Boolean					fHasScrollingFocus;		// TRUE when frame has the scrolling focus
	ODFacet*					fActiveFacet;			// Active facet for this frame
	FW_Boolean					fCanBeActiveFrame;		// FALSE for floating window frames
	FW_CPresentation*			fPresentation;
	FW_CWindow*					fWindow;
	unsigned short				fFacetCount;
	ODTypeToken					fPreviousViewType;
	FW_CViewAs*					fViewAs;
	FW_CIdler*					fIdler;
	
	FW_MDraggableFrame*			fDraggable;
	FW_MDroppableFrame*			fDroppable;
	FW_DroppableState			fDroppableState;
	
	FW_CPrivBaseScroller*		fScroller;
	
	FW_CView*					fViewUnderCursor;		// To handle MouseEnter/Within/Leave events
	FW_Boolean					fRequestedFrame;		// TRUE if the frame has been gotton thru RequestEmbeddedFrame
	FW_Boolean					fInSharedWindow;		// [HLX] hack for shared window
#ifdef FW_BUILD_WIN
	ODPlatformWindow			fShadowWindow;
#endif
	FW_Boolean					fInternalTransformRealigmentNeeded;
	FW_CPoint					fContentViewOffset;		// content view location in frame coordinates
	FW_Boolean					fSubViewsCreated;		// FALSE initialy. TRUE if the subviews are created.
	FW_CFrame*					fSourceFrame;			
	FW_Boolean					fInfoCommandAdjusted;	// true if PartInfo menu item was already set up
	FW_Boolean					fShowPaletteOnMouseUp;	// Set to TRUE in FocusStateChanged	
	ODShape*					fContentShape;			// cached content shape
	FW_Boolean					fAdoptsContainerProperties;	// true if adopts ContainerPartProperties
};


//========================================================================================
//	Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CFrame::PrivSetInfoCommandAdjusted
//----------------------------------------------------------------------------------------
inline void FW_CFrame::PrivSetInfoCommandAdjusted(Environment*, FW_Boolean value)
{
	fInfoCommandAdjusted = value;
}

//----------------------------------------------------------------------------------------
// FW_CFrame::PrivGetContentViewOffset
//----------------------------------------------------------------------------------------
inline FW_CPoint FW_CFrame::PrivGetContentViewOffset(Environment*) const
{
	return fContentViewOffset;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetDraggable
//----------------------------------------------------------------------------------------
inline FW_MDraggableFrame* FW_CFrame::GetDraggable(Environment*) const
{
	return fDraggable;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetDroppable
//----------------------------------------------------------------------------------------
inline FW_MDroppableFrame* FW_CFrame::GetDroppable(Environment*) const
{
	return fDroppable;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivSetDraggable
//----------------------------------------------------------------------------------------
inline void FW_CFrame::PrivSetDraggable(Environment*, FW_MDraggableFrame* draggable)
{
	fDraggable = draggable;
}
	
//----------------------------------------------------------------------------------------
//	FW_CFrame::GetFacetCount
//----------------------------------------------------------------------------------------
inline unsigned short FW_CFrame::GetFacetCount(Environment*) const
{
	return fFacetCount;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivInSharedWindow
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::PrivInSharedWindow(Environment*) const
{
	return fInSharedWindow;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetFrontClicks
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::GetFrontClicks(Environment*) const
{
	return fGetFrontClicks;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetFrontClicks
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetFrontClicks(Environment*, FW_Boolean getFrontClicks)
{
	fGetFrontClicks = getFrontClicks;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetPart
//----------------------------------------------------------------------------------------
inline FW_CPart* FW_CFrame::GetPart(Environment*) const
{
	return fPart;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetScroller
//----------------------------------------------------------------------------------------
inline FW_CScroller* FW_CFrame::GetScroller(Environment*) const
{
	return (FW_CScroller*)fScroller;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetContentView
//----------------------------------------------------------------------------------------
inline FW_CSuperView* FW_CFrame::GetContentView(Environment*) const
{
		// cast away const
	return (fContentView == NULL) ? (FW_CSuperView*)this : fContentView;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetActiveFacet
//----------------------------------------------------------------------------------------
inline ODFacet* FW_CFrame::GetActiveFacet(Environment*) const
{
	return fActiveFacet;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetCanBeActiveFrame
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetCanBeActiveFrame(Environment*, FW_Boolean beActive)
{
	fCanBeActiveFrame = beActive;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::ChangeUsedShape
//----------------------------------------------------------------------------------------
inline void FW_CFrame::ChangeUsedShape(Environment* ev, ODShape* newUsedShape, ODCanvas* biasCanvas)
{
	fODFrame->ChangeUsedShape(ev, newUsedShape, biasCanvas);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetODFrame
//----------------------------------------------------------------------------------------
inline ODFrame* FW_CFrame::GetODFrame(Environment*) const
{
	return fODFrame;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivGetPreviousViewType
//----------------------------------------------------------------------------------------
inline ODTypeToken FW_CFrame::PrivGetPreviousViewType(Environment*) const
{
	return fPreviousViewType;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivSetPreviousViewType
//----------------------------------------------------------------------------------------
inline void FW_CFrame::PrivSetPreviousViewType(Environment*, ODTypeToken viewType)
{
	fPreviousViewType = viewType;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetFrameGroup
//----------------------------------------------------------------------------------------
inline ODID FW_CFrame::GetFrameGroup(Environment* ev) const
{
	return fODFrame->GetFrameGroup(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetSequenceNumber
//----------------------------------------------------------------------------------------
inline ODID FW_CFrame::GetSequenceNumber(Environment* ev) const
{
	return fODFrame->GetSequenceNumber(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsRoot
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsRoot(Environment* ev) const
{
	return fODFrame->IsRoot(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::HasSelectionFocus
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::HasSelectionFocus(Environment*) const
{
	return fHasSelectionFocus;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsPersistent
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsPersistent(Environment* ev) const
{
	return fODFrame->GetStorageUnit(ev) != NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsOverlaid
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsOverlaid(Environment* ev) const
{
	return fODFrame->IsOverlaid(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsInLimbo
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsInLimbo(Environment* ev) const
{
	return FW_IsInLimbo(ev, fODFrame);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsFrozen
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsFrozen(Environment* ev) const
{
	return fODFrame->IsFrozen(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetFrozen
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetFrozen(Environment* ev, FW_Boolean isFrozen)
{
	fODFrame->SetFrozen(ev, isFrozen);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::DoesPropagateEvents
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::DoesPropagateEvents(Environment* ev) const
{
	return fODFrame->DoesPropagateEvents(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetPropagateEvents
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetPropagateEvents(Environment* ev, FW_Boolean doesPropagateEvents)
{
	fODFrame->SetPropagateEvents(ev, doesPropagateEvents);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetViewType
//----------------------------------------------------------------------------------------
inline ODTypeToken FW_CFrame::GetViewType(Environment* ev) const
{
	return fODFrame->GetViewType(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetPresentation
//----------------------------------------------------------------------------------------
inline FW_CPresentation* FW_CFrame::GetPresentation(Environment*) const
{
	return fPresentation;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsSubframe
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsSubframe(Environment* ev) const
{
	return fODFrame->IsSubframe(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetSubframe
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetSubframe(Environment* ev, FW_Boolean isSubframe)
{
	fODFrame->SetSubframe(ev, isSubframe);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AcquireUsedShape
//----------------------------------------------------------------------------------------
inline ODShape* FW_CFrame::AcquireUsedShape(Environment* ev, ODCanvas* biasCanvas) const
{
	return fODFrame->AcquireUsedShape(ev, biasCanvas);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AcquireFrameShape
//----------------------------------------------------------------------------------------
inline ODShape* FW_CFrame::AcquireFrameShape(Environment* ev, ODCanvas* biasCanvas) const
{
	return fODFrame->AcquireFrameShape(ev, biasCanvas);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AcquireContainingFrame
//----------------------------------------------------------------------------------------
inline ODFrame* FW_CFrame::AcquireContainingFrame(Environment* ev) const
{
	return fODFrame->AcquireContainingFrame(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AcquireInternalTransform
//----------------------------------------------------------------------------------------
inline ODTransform* FW_CFrame::AcquireInternalTransform(Environment* ev, ODCanvas* biasCanvas) const
{
	return fODFrame->AcquireInternalTransform(ev, biasCanvas);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AcquireODWindow
//----------------------------------------------------------------------------------------
inline ODWindow* FW_CFrame::AcquireODWindow(Environment* ev) const
{
	return fODFrame->AcquireWindow(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetID
//----------------------------------------------------------------------------------------
inline ODID FW_CFrame::GetID(Environment*) const
{
	return fFrameID;
}

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CFrame::GetShadowWindow
//----------------------------------------------------------------------------------------
inline ODPlatformWindow FW_CFrame::GetShadowWindow(Environment*) const
{
	return fShadowWindow;
}
#endif

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetAdoptsContainerProperties
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::GetAdoptsContainerProperties(Environment*) const
{
	return fAdoptsContainerProperties;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetAdoptsContainerProperties
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetAdoptsContainerProperties(Environment*, FW_Boolean adoptsProperties)
{
	fAdoptsContainerProperties = adoptsProperties;
}

//----------------------------------------------------------------------------------------
// FW_CFrame::PrivSetRequestedFrame
//----------------------------------------------------------------------------------------
inline void FW_CFrame::PrivSetRequestedFrame(Environment*)
{
	fRequestedFrame = TRUE;
}

//----------------------------------------------------------------------------------------
// FW_CFrame::IsRequestedFrame
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsRequestedFrame(Environment*) const
{
	return fRequestedFrame;
}

//----------------------------------------------------------------------------------------
// FW_CFrame::GetSourceFrame
//----------------------------------------------------------------------------------------
inline FW_CFrame* FW_CFrame::GetSourceFrame(Environment*)
{
	return fSourceFrame;
}

//----------------------------------------------------------------------------------------
// FW_CFrame::PrivGetDroppableState
//----------------------------------------------------------------------------------------
inline FW_DroppableState FW_CFrame::PrivGetDroppableState() const
{
	return fDroppableState;
}

//----------------------------------------------------------------------------------------
// FW_CFrame::PrivSetDroppableState
//----------------------------------------------------------------------------------------
inline void FW_CFrame::PrivSetDroppableState(FW_DroppableState droppableState)
{
	fDroppableState = droppableState;
}

//----------------------------------------------------------------------------------------
// FW_CFrame::PrivSubViewsCreated
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::PrivSubViewsCreated(Environment*)
{
	return fSubViewsCreated;
}

#endif

