//========================================================================================
//
//	File:				FWKind.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWKIND_H
#include "FWKind.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWCOMMON_H
#include "FWCommon.h"
#endif

#ifndef FWSESION_H
#include "FWSesion.h"
#endif

#ifndef FWMEMMGR_H
#include "FWMemMgr.h"
#endif

#ifndef FWACQUIR_H
#include "FWAcquir.h"
#endif

#ifndef SOM_ODTranslation_xh
#include "Translt.xh"
#endif

//========================================================================================
//	Template Instantiations
//========================================================================================

FW_DEFINE_AUTO_TEMPLATE(FW_TOrderedCollectionIterator, FW_CKind)
FW_DEFINE_AUTO_TEMPLATE(FW_TOrderedCollection, FW_CKind)

#ifdef FW_USE_TEMPLATE_PRAGMAS

#pragma template_access public
#pragma template FW_TOrderedCollection<FW_CKind>
#pragma template FW_TOrderedCollectionIterator<FW_CKind>

#endif

//========================================================================================
//	class FW_CKind
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CKind::FW_CKind
//----------------------------------------------------------------------------------------

FW_CKind::FW_CKind(Environment* ev, FW_CPart* part, ODType kind, FW_StorageKinds storageKind, FW_ImportExport importExport) :
	fPart(part),
	fType(NULL),
	fStorageKind(storageKind),
	fImportExport(importExport),
	fIsPartKind(FALSE),
	fIsPreferredKind(FALSE),
	fDirty(0)
{
FW_UNUSED(ev);

	fType = FW_PrimitiveStringDuplicate(kind);
}

//----------------------------------------------------------------------------------------
// FW_CKind::~FW_CKind
//----------------------------------------------------------------------------------------

FW_CKind::~FW_CKind()
{
	if (fType)
	{
		FW_CMemoryManager::FreeBlock(fType);
		fType = NULL;
	}
}

//----------------------------------------------------------------------------------------
// FW_CKind::ChangeStorageKind
//----------------------------------------------------------------------------------------

void FW_CKind::ChangeStorageKind(Environment* ev, FW_StorageKinds storageKind)
{
	// Cannot allow to remove FW_kPartStorage to the Part kind.
#ifdef FW_DEBUG
	if (IsPartKind(ev) && ((storageKind & FW_kPartStorage) == 0))
	{
		FW_DEBUG_MESSAGE("FW_CKind::ChangeStorageKind - Can't remove part storage to part kind");
		return;
	}
#endif

	// ----- If changes in Part Storage I need to reprepare the kODPropContents -----
	if (((fStorageKind & FW_kPartStorage) != (storageKind & FW_kPartStorage)) && ToBeExported(ev, FW_kPartStorage))
		fPart->PrivNeedPrepping(ev, TRUE);
		
	fStorageKind = storageKind;

	if (ToBeExported(ev, fStorageKind))
		PrivSetDirty(ev, TRUE, fStorageKind);
}

//----------------------------------------------------------------------------------------
// FW_CKind::ChangeImportExport
//----------------------------------------------------------------------------------------

void FW_CKind::ChangeImportExport(Environment* ev, FW_ImportExport importExport)
{
#ifdef FW_DEBUG
	if (IsPartKind(ev))
		FW_DEBUG_MESSAGE("FW_CKind::ChangeImportExport - Can't change the import/export of the part Kind");
	else
#endif
	if (fImportExport != importExport)
	{
		fImportExport = importExport;
		
		if (ToBeExported(ev, FW_kPartStorage))
			fPart->PrivNeedPrepping(ev, TRUE);
	}
}

//----------------------------------------------------------------------------------------
// FW_CKind::IsEqual
//----------------------------------------------------------------------------------------

FW_Boolean FW_CKind::IsEqual(Environment* ev, ODType kind) const
{
FW_UNUSED(ev);

	return FW_PrimitiveStringEqual(fType, kind);
}

//----------------------------------------------------------------------------------------
// FW_CKind::IsEqual
//----------------------------------------------------------------------------------------

FW_Boolean FW_CKind::IsEqual(Environment* ev, ODPlatformType platformType, ODPlatformTypeSpace typeSpace) const
{
	ODTranslation *translate = FW_CSession::GetTranslation(ev);
	FW_CAcquireODISOStr type = translate->GetISOTypeFromPlatformType(ev, platformType, typeSpace);

	return FW_PrimitiveStringEqual(fType, type);
}

//----------------------------------------------------------------------------------------
// FW_CKind::PrivSetDirty
//----------------------------------------------------------------------------------------

void FW_CKind::PrivSetDirty(Environment* ev, FW_Boolean state, FW_StorageKinds theStorage)
{
FW_UNUSED(ev);

	if (state)
		fDirty |= theStorage;	// set the bit(s)
	else
		fDirty ^= (fDirty & theStorage);	// clear the bit(s)
}
