//========================================================================================
//
//	File:				FWODPart.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWODPART_H
#define FWODPART_H

#ifndef SOM_ODStorageUnitView_xh
#include <SUView.xh>
#endif

#ifndef SOM_ODCanvas_xh
#include <Canvas.xh>
#endif

#ifndef SOM_ODDragItemIterator_xh
#include <DgItmIt.xh>
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class ODFrame;
class ODWindow;
class ODSession;
class ODTypeList;
class ODLinkSource;
class ODObjectSpec;
class ODEmbeddedFramesIterator;
class ODLink;

class FW_CEmbeddingPart;
class FW_CPoint;
class FW_CPart;

//========================================================================================
//	class FW_CODPart
//========================================================================================

class FW_CODPart
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
protected:
	FW_CODPart() {}
	~FW_CODPart() {}

//----------------------------------------------------------------------------------------
//	ODPart API
//
public:
	//--------------------------------------------------------------------------
	//	Initialization
	
	static void				InitPart(Environment *ev,
									FW_CPart* part,
									ODStorageUnit* storageUnit);

	static void				InitPartFromStorage(Environment *ev,
									FW_CPart* part,
									ODStorageUnit* storageUnit);

	//--------------------------------------------------------------------------
	//	From Layout protocol

	static void				DisplayFrameAdded(Environment *ev,  
								FW_CPart* part,
								ODFrame* odFrame);

	static void				DisplayFrameRemoved(Environment *ev,
									FW_CPart* part,
									ODFrame* odFrame,
									FW_Boolean toStorage);

	static void				DisplayFrameConnected(Environment *ev, 
								ODFrame* odFrame);

	static void				AttachSourceFrame(Environment *ev,
								ODFrame* odFrame,
								ODFrame* odSourceFrame);

	static void				FrameShapeChanged(Environment *ev,  
								ODFrame* odFrame);
	
	static void				ViewTypeChanged(Environment *ev,  
								ODFrame* odFrame);

	static void				PresentationChanged(Environment *ev,  
								ODFrame* odFrame);

	static void				SequenceChanged(Environment *ev,  
								ODFrame* odFrame);
	
	static void				LinkStatusChanged(Environment *ev,  
								FW_CPart* part,
								ODFrame* odFrame);
	
	static void				ContainingPartPropertiesUpdated(Environment *ev,
								ODFrame* odFrame,
								ODStorageUnit* propertyUnit);

	static ODInfoType		ReadPartInfo(Environment *ev,
								FW_CPart* part,
								ODFrame* odFrame,
								ODStorageUnitView* storageUnitView);

	static void				WritePartInfo(Environment *ev,
								ODInfoType partInfo,
								ODStorageUnitView* storageUnitView);

	static void				ClonePartInfo(Environment *ev,
								ODDraftKey key,
								ODInfoType partInfo,
								ODStorageUnitView* storageUnitView,
								ODFrame* scope);

	static ODID				Open(Environment *ev,  
								FW_CPart* part,
								ODFrame* odFrame);

	//--------------------------------------------------------------------------
	//	From Imaging protocol

	static void				Draw(Environment *ev,
								ODFacet* facet,
								ODShape* invalidShape);

	static void				FacetAdded(Environment *ev,  
								ODFacet* facet);

	static void				FacetRemoved(Environment *ev,  
								ODFacet* facet);

	static void				GeometryChanged(Environment *ev,
								ODFacet* facet,
								FW_Boolean clipShapeChanged,
								FW_Boolean externalTransformChanged);

	static void				HighlightChanged(Environment *ev,  
								ODFacet* facet);

	static void				CanvasChanged(Environment *ev,  
								ODFacet* facet);

	static void				CanvasUpdated(Environment *ev, 
								ODCanvas* canvas);

	static unsigned long	GetPrintResolution(Environment *ev, 
								ODFrame* odFrame);

	//--------------------------------------------------------------------------
	// From Part Activation protocol

	static FW_Boolean 		BeginRelinquishFocus(Environment *ev,
								FW_CPart* part,
								ODTypeToken focus,
								ODFrame* odOwnerFrame,
								ODFrame* proposedFrame);
	
	static void 			CommitRelinquishFocus(Environment *ev,
								ODTypeToken focus,
								ODFrame* odOwnerFrame,
								ODFrame* odProposedFrame);
	              
	static void 			AbortRelinquishFocus(Environment *ev,
								ODTypeToken focus,
								ODFrame* ownerFrame,
								ODFrame* proposedFrame);
	                  
	static void				FocusAcquired(Environment *ev,
								ODTypeToken focus,
								ODFrame* ownerFrame);
	  
	static void				FocusLost(Environment *ev,
								ODTypeToken focus,
								ODFrame* ownerFrame);
          

	//--------------------------------------------------------------------------
	// From Part Persistance protocol

	static void				ExternalizeKinds(Environment *ev,
								FW_CPart* part, 
								ODTypeList* kindset);
	
	static void				ChangeKind(Environment *ev, 
								FW_CPart* part,
								ODType kind);

	//--------------------------------------------------------------------------
	// From UI Events protocol

	static FW_Boolean		HandleEvent(Environment *ev,
								FW_CPart* part,
								ODEventData* event,
								ODFrame* odFrame,
								ODFacet* odFacet,
								ODEventInfo* eventInfo);
  
	static void				AdjustMenus(Environment *ev, 
								FW_CPart* part,
								ODFrame* odFrame);
    
	//--------------------------------------------------------------------------
	// From Undo protocol

	static void				UndoAction(Environment *ev, 
								FW_CPart* part,
								ODActionData* actionState);
	
	static void				RedoAction(Environment *ev, 
								ODActionData* actionState);
	
	static void				DisposeActionState(Environment *ev,
								ODActionData* actionState,
								ODDoneState doneState);
	
	static void				WriteActionState(Environment *ev,
								ODActionData* actionState,
								ODStorageUnitView* storageUnitView);
	
	static void				ReadActionState(Environment *ev, 
								ODStorageUnitView* storageUnitView,
								ODActionData* actionData);

	//--------------------------------------------------------------------------
	// From DragAndDrop protocol

	static void				FulfillPromise(Environment *ev, 
								FW_CPart* part, 
								ODStorageUnitView* promiseSUView);
	
	static void				DropCompleted(Environment *ev,
								ODPart* destPart,
								ODDropResult dropResult);
	
	static ODDragResult		DragEnter(Environment *ev,
								ODDragItemIterator* dragInfo,
								ODFacet* facet,
								const FW_CPoint& where);
	
	static ODDragResult		DragWithin(Environment *ev,
								ODDragItemIterator* dragInfo,
								ODFacet* facet,
								const FW_CPoint& where);
	
	static void				DragLeave(Environment *ev,
								ODFacet* facet,
								const FW_CPoint& where);
	
	static ODDropResult		Drop(Environment *ev,
								ODDragItemIterator* dropInfo,
								ODFacet* facet,
								const FW_CPoint& where);

	//--------------------------------------------------------------------------
	// From Linking protocol

	static ODLinkSource* 	CreateLink(Environment *ev, 
								FW_CPart* part,
								ODByteArray* data);
	
	static void				RevealLink(Environment *ev, 
								FW_CPart* part,
								ODLinkSource* linkSource);
	
	static void				LinkUpdated(Environment *ev,
								FW_CPart* part, 
								ODLink* updatedLink,
								ODUpdateID updateID);
	
	// --- for containing parts ---
	
	static void				EmbeddedFrameUpdated(Environment *ev,
								FW_CPart* part,
								ODFrame* odFrame,
								ODUpdateID updateID);
	
	static FW_Boolean		EditInLinkAttempted(Environment *ev, 
								FW_CPart* part,
								ODFrame* odFrame);

	//--------------------------------------------------------------------------
	// From Embedding protocol

	static ODFrame*			RequestEmbeddedFrame(Environment *ev,
								FW_CEmbeddingPart* part,
								ODFrame* containingFrame,
								ODFrame* baseFrame,
								ODShape* suggestedFrameShape,
								ODPart* embeddedPart,
								ODTypeToken viewType,
								ODTypeToken presentation,
								FW_Boolean isOverlaid);
	
	static void				RemoveEmbeddedFrame(Environment *ev,
								FW_CEmbeddingPart* part,
								ODFrame* embeddedFrame);
	
	static ODShape*			RequestFrameShape(Environment *ev,
								ODFrame* embeddedFrame,
								ODShape* frameShape);
	
	static void				UsedShapeChanged(Environment *ev,
								ODFrame* embeddedFrame);
	
	static ODShape*			AdjustBorderShape(Environment *ev,
								ODFacet* embeddedFacet,
								ODShape* shape);

	static ODStorageUnit*	AcquireContainingPartProperties(Environment *ev,
								FW_CEmbeddingPart* part,
								ODFrame* embeddedFrame);

	static FW_Boolean		RevealFrame(Environment *ev,
								FW_CEmbeddingPart* part,
								ODFrame* embeddedFrame,
								ODShape* revealShape);
        
	//--------------------------------------------------------------------------
	// Inherited ODPart API

	static void				Externalize(Environment *ev, FW_CPart* part);
	static void 			CloneInto(Environment *ev,
								FW_CPart* part,
								ODDraftKey key,
								ODStorageUnit* toSU,
								ODFrame* scope);

	static void				Release(Environment *ev, FW_CPart* part);
	static void				ReleaseAll(Environment *ev, FW_CPart* part);
	
	static ODSize			Purge(Environment *ev, FW_CPart*, ODSize size);

};

#endif



