//========================================================================================
//
//	File:				FWPxyFrm.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPXYFRM_H
#define FWPXYFRM_H

#ifndef FWODMISS_H
#include "FWODMiss.h"
#endif

#ifndef FWTCOLL_H
#include "FWTColl.h"
#endif

#ifndef FWODTYPS_H
#include "FWODTyps.h"
#endif

#ifndef FWPRTDEF_H
#include "FWPrtDef.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class ODFrame;
class ODCanvas;
class ODDraft;
class ODStorageUnitView;
class ODTransform;

class FW_CCloneInfo;
class FW_MProxy;
class FW_CEmbeddingFrame;
class FW_CEmbeddingPart;
class FW_CRect;

//========================================================================================
//	class FW_CProxyFrame
//========================================================================================

class FW_CProxyFrame
{
	friend class FW_MProxy;
	
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
protected:
	FW_CProxyFrame(Environment* ev, 
				FW_MProxy* proxy, 
				FW_CEmbeddingPart* part);
	FW_CProxyFrame(Environment* ev, 
				FW_MProxy* proxy, 
				FW_CEmbeddingPart* part, 
				FW_CEmbeddingFrame* containingFrame, 
				ODFrame* embeddedFrame,
				FW_Boolean wasInLimbo);
	~FW_CProxyFrame();
	
//----------------------------------------------------------------------------------------
//	Public API
//
public:
	FW_CEmbeddingFrame*	GetContainingFrame(Environment* ev) const;
	ODStorageUnitID		GetContainingFrameID(Environment* ev) const;
	
	FW_MProxy*			GetProxy(Environment* ev) const;
	
	FW_Boolean			IsFrameInMemory(Environment* ev) const;
	ODFrame*			GetFrame(Environment* ev) const;

	ODStorageUnitID		GetFrameID(Environment* ev) const;

	FW_Boolean			IsAttached(Environment* ev) const;
	
	// ----- Purge -----
	FW_Boolean			IsPurgeable(Environment* ev) const;
	void				SetPurgeable(Environment* ev, FW_Boolean purgeable);
	
	// ----- Add/Remove all facets of a proxyFrame (hide/show) -----
	void				RemoveAllFacets(Environment* ev);
	void				CreateAllFacets(Environment* ev);								

	// ----- hilite -----
	void				ChangeHighlight(Environment* ev, ODHighlight highlight);
	ODHighlight			GetHighlight(Environment* ev) const;
	
//----------------------------------------------------------------------------------------
//	Internal API
//
public:	
	void				SetContainingFrame(Environment* ev, FW_CEmbeddingFrame* containingFrame);
	void				SetProxy(Environment* ev, FW_MProxy* proxy);
	
	void				Release(Environment* ev);
	FW_Boolean			IsOrphan(Environment* ev) const;
	
	// ----- Facet creation and removal -----		
	void				RemoveFacets(Environment* ev, FW_CEmbeddingFrame* frame, ODFacet* embeddingFacet);
	void				CreateFacets(Environment* ev, FW_CEmbeddingFrame* frame, ODFacet* embeddingFacet);									

	// ----- Purge -----
	ODSize				Purge(Environment* ev);			

	// ----- Persistence -----
	FW_Boolean			InternalizeProxyFrame(Environment* ev, ODStorageUnitView* suView, FW_CCloneInfo* cloneInfo);
	FW_Boolean			ExternalizeProxyFrame(Environment* ev, ODStorageUnitView* suView, FW_CCloneInfo* cloneInfo);
	
//----------------------------------------------------------------------------------------
//	Internal API
//
public:	
	void				PrivClose(Environment* ev);

	void				PrivAttach(Environment* ev);	
	void				PrivDetach(Environment* ev);	
	
	void				PrivSetFrameGroup(Environment* ev, ODID frameGroup);
	void				PrivChangeSequenceNumber(Environment* ev, ODID sequenceNumber);

	void				PrivSetDragging(Environment* ev, FW_EPrivDraggingFlags dragFlags);
	void				PrivCreateFacets(Environment* ev, FW_CEmbeddingFrame* frame, ODFacet* embeddingFacet);

	FW_Boolean			PrivNeedFacets(Environment* ev, FW_CEmbeddingFrame* embeddingFrame);
	void  				ChangeLinkStatus(Environment* ev, ODLinkStatus linkStatus);

private:
	void  				PrivChangeLinkStatus(Environment* ev) const;

	
//----------------------------------------------------------------------------------------
//	Data Members
//
private:	
	FW_CEmbeddingPart*	fPart;
	
	ODStorageUnitID 	fFrameID;	
	ODDraft*			fFrameDraft;
		
	ODStorageUnitID		fContainingFrameID;

	FW_MProxy*			fProxy;
	ODFrame* 			fEmbeddedFrame;
		
	FW_Boolean			fPurgeable;
	FW_Boolean			fHasBeenInstantiated;	// TRUE: the ODFrame has been instantiated at least once.
	FW_Boolean			fAttached;				// Used by FW_MProxy::Attach/DetachEmbeddedFrames
	
	FW_Boolean			fWasInLimbo;
	FW_Boolean			fWasDragged;
	unsigned short		fFacetCount;
	
	ODHighlight 		fHighlight;
	ODLinkStatus		fLinkStatus;
};


//========================================================================================
// Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::IsPurgeable
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CProxyFrame::IsPurgeable(Environment*) const
{
	return fPurgeable;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::IsFrameInMemory
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CProxyFrame::IsFrameInMemory(Environment*) const
{
	return fEmbeddedFrame != NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::GetFrameID
//----------------------------------------------------------------------------------------
inline ODStorageUnitID FW_CProxyFrame::GetFrameID(Environment*) const
{
	return fFrameID;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::GetProxy
//----------------------------------------------------------------------------------------
inline FW_MProxy* FW_CProxyFrame::GetProxy(Environment*) const
{
	return fProxy;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::GetContainingFrameID
//----------------------------------------------------------------------------------------
inline ODStorageUnitID FW_CProxyFrame::GetContainingFrameID(Environment*) const
{
	return fContainingFrameID;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::IsOrphan
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CProxyFrame::IsOrphan(Environment*) const
{
	return fContainingFrameID == kODNULLID;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::IsAttached
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CProxyFrame::IsAttached(Environment*) const
{
	return fAttached;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::GetHighlight
//----------------------------------------------------------------------------------------
inline ODHighlight FW_CProxyFrame::GetHighlight(Environment*) const
{
	return fHighlight;
}

#endif
