//========================================================================================
//
//	File:				FWScrolr.h
//	Release Version:	$ ODF 2 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWSCROLR_H
#define FWSCROLR_H

#ifndef FWSVIEW_H
#include "FWSView.h"
#endif

#ifndef FWCONTRL_H
#include "FWContrl.h"
#endif

// ----- OS Layer -----

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

// ----- Foundation Layer -----

#ifndef FWRECEVR_H
#include "FWRecevr.h"
#endif

//========================================================================================
// Forward declarations
//========================================================================================

class FW_CPoint;
class FW_CFrame;
class FW_CScrollBar;

//========================================================================================
// CLASS FW_CPrivBaseScroller
//========================================================================================
// This class is only here for better dead-stripping. Because FW_CFrame has to call some of the
// FW_CScroller's method I don't want the whole FW_CScroller class to be dragged in.

class FW_CPrivBaseScroller
{
public:
	FW_DECLARE_CLASS

	FW_CPrivBaseScroller();
	virtual ~FW_CPrivBaseScroller();

public:
	virtual void				InitializeAutoScroll(Environment* ev) = 0;
	virtual FW_CPoint			AutoScrollOffset(Environment* ev, 
									const FW_CPoint& currentPoint,
									unsigned long delay = 0) = 0;
	virtual void				ScrollBy(Environment* ev, 
									const FW_CPoint& offset,
									FW_CGraphicContext* gc = NULL) = 0;

	virtual void				UpdateScrollParameters(Environment* ev) = 0;

	// ----- PageNavigation Key -----
	FW_Boolean					IsHandlingPageNavigationKeys(Environment* ev) const;
	virtual FW_Handled			HandlePageNavigationKeys(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent) = 0;
	
	// ----- Archiving -----
	static void* 				Create(FW_CReadableStream& stream, FW_ClassTypeConstant type);
	static void					Destroy(void* object, FW_ClassTypeConstant type);
	static void					Read(FW_CReadableStream& stream, FW_ClassTypeConstant type, void* object);
	static void 				Write(FW_CWritableStream& stream, FW_ClassTypeConstant type, const void *object);

	// ----- Internal API -----
	virtual void				PrivRealignScroller(Environment* ev) = 0;

#ifdef FW_DEBUG
public:
	virtual void				PrivCheckFrame(Environment* ev) const;
#endif	

protected:
	FW_Boolean 					fHandlePageNavigationKeys;			// if it responds to PgUp/ PgDn, etc.
};

//========================================================================================
// CLASS FW_CScroller
//========================================================================================

class FW_CScroller : public FW_CPrivBaseScroller, public FW_MReceiver
{
public:
	FW_DECLARE_CLASS
	FW_DECLARE_AUTO(FW_CScroller)

	enum EAutoScrollStage
	{
		kOutStage 		= 0,
		kDelayStage 	= 1,
		kScrollStage 	= 2
	};
	
// ----- Constructors/Destructors
public:
	FW_CScroller(Environment* ev, FW_CFrame* frame);
	virtual ~FW_CScroller();
		
protected:
	FW_CScroller(Environment* ev);

public:
	// ----- Notification -----
	
	void						RegisterScrollNotifier(Environment* ev, 
									FW_MNotifier* notifier);
	void						UnregisterScrollNotifier(Environment* ev, 
									FW_MNotifier* notifier);
	virtual void				HandleNotification(Environment* ev, 
									const FW_CNotification& notification);
	
	// ----- Srolling -----
	void						ScrollBy(Environment* ev, 
									const FW_CPoint& offset,
									FW_CGraphicContext* gc = NULL);
	void						ScrollTo(Environment* ev, 
									const FW_CPoint& position, 
									FW_CGraphicContext* gc = NULL);
	
	void						RevealRect(Environment* ev, 
									 const FW_CRect& rectToReveal, 
							 		FW_CGraphicContext* gc = NULL);
				
	// ----- AutoScrolll -----
	virtual void				InitializeAutoScroll(Environment* ev);
	virtual FW_CPoint			AutoScrollOffset(Environment* ev, 
									const FW_CPoint& currentPoint,
									unsigned long delay = 0);
	void						AutoScroll(Environment* ev, 
									const FW_CPoint& previousPoint, 
									FW_CGraphicContext* gc = NULL,
									unsigned long delay = 0);
	void						SetAutoScrollInset(Environment* ev, 
									FW_Fixed inset);

	// ----- Scalling -----
	void						ScaleBy(Environment* ev, 
									const FW_CPoint& scaling,
									FW_CGraphicContext* gc = NULL);

	// ----- Notification -----
	virtual void				ScrollPositionChanged(Environment* ev, 
									FW_Fixed oldPos, 
									FW_Fixed newPos, 
									FW_XYSelector direction);

	// ----- Scroller Adjustments Parameters -----
	virtual FW_CPoint			TranslationToPixels(Environment* ev, const FW_CPoint& translation) const;

	virtual FW_Fixed			CalcMaxTranslation(Environment* ev, FW_Fixed scrollingArea, FW_XYSelector direction);
	virtual FW_CRect			GetScrollingArea(Environment* ev) const;
										
	virtual void				UpdateScrollParameters(Environment* ev);
	
	// ----- Scroll Units -----
	virtual FW_Fixed			GetPageUnit(Environment* ev, FW_XYSelector direction) const;
	FW_Fixed					GetScrollUnit(Environment* ev, FW_XYSelector direction) const;
	
	void						SetScrollUnits(Environment* ev, const FW_CPoint& scrollUnit);
	FW_CPoint					GetScrollUnits(Environment* ev) const;
	
	// ----- PageNavigation Key -----
	void						ChangeHandlingOfPageNavigationKeys(Environment* ev, FW_Boolean state);
	virtual FW_Handled			HandlePageNavigationKeys(Environment* ev, 
												const FW_CVirtualKeyEvent& theVirtualKeyEvent);
	
	// ----- Archiving -----
	static void* 				Create(FW_CReadableStream& stream, FW_ClassTypeConstant type);
	static void					Destroy(void* object, FW_ClassTypeConstant type);
	static void					Read(FW_CReadableStream& stream, FW_ClassTypeConstant type, void* object);
	static void 				Write(FW_CWritableStream& stream, FW_ClassTypeConstant type, const void *object);

	virtual void				Flatten(Environment*ev, FW_CWritableStream& stream) const;
	virtual void				InitializeFromStream(Environment*ev, FW_CReadableStream& stream);

//----------------------------------------------------------------------------------------
//	Internal API
//
public:	
	// ----- Internal API -----
	virtual void				PrivRealignScroller(Environment* ev);

	virtual void				PrivAdjustTranslations(Environment* ev, 
													FW_XYSelector direction, 
													FW_Fixed translation,
													FW_Fixed scrollingArea);

protected:
	void						PrivScrollDraw(Environment* ev, 
									FW_Fixed by, 
									FW_XYSelector direction);
	
	void						PrivChangeFrameInternalTransform(Environment* ev, 
									const FW_CPoint& odlTranslation, 
									const FW_CPoint& newTranslation, 
									FW_XYSelector direction,
									FW_Boolean scroll);
	ODShape*					PrivCreateContentScrollShape(Environment* ev, 
									FW_XYSelector direction) const;
	
	void						PrivScrollBy(Environment* ev, const FW_CPoint& offset, FW_CGraphicContext* gc);

	void						PrivLimitScrolling(FW_CPoint& scrollBy) const;

//----------------------------------------------------------------------------------------
//	Data Members
//
protected:
	FW_CFrame*				fFrame;
	FW_Fixed				fAutoScrollInset;
	FW_CPoint				fTranslation;
    FW_CPoint				fMaxTranslation;	

private:
	EAutoScrollStage		fAutoScrollStage;
	unsigned long			fLastAutoScrollTick;
	
	FW_CPoint				fScrollUnit;		// increment of scroll
};

//========================================================================================
// CLASS FW_CScrollBarScroller
//========================================================================================

class FW_CScrollBarScroller : public FW_CScroller
{
public:
	FW_DECLARE_CLASS
	FW_DECLARE_AUTO(FW_CScrollBarScroller)

// ----- Constructors/Destructors
public:
	FW_CScrollBarScroller(Environment* ev,
						  FW_CFrame* frame, 
						  FW_Boolean liveScrolling,
						  FW_CScrollBar* horzSB, 
						  FW_CScrollBar* vertSB);
	virtual ~FW_CScrollBarScroller();

protected:
	FW_CScrollBarScroller(Environment* ev);

public:
	virtual void			ScrollPositionChanged(Environment* ev, 
									FW_Fixed oldPos, 
									FW_Fixed newPos, 
									FW_XYSelector direction);	
	
	FW_CScrollBar*			GetScrollBar(Environment* ev, FW_XYSelector direction) const;
	virtual void			RemoveScrollBar(Environment* ev, FW_XYSelector direction);
	
	// ----- Archiving -----
	static void* 			Create(FW_CReadableStream& stream, FW_ClassTypeConstant type);
	static void				Destroy(void* object, FW_ClassTypeConstant type);
	virtual void			Flatten(Environment*ev, FW_CWritableStream& stream) const;
	virtual void			InitializeFromStream(Environment*ev, FW_CReadableStream& stream);
	
//----------------------------------------------------------------------------------------
//	Internal API
//
public:	
	virtual void			PrivAdjustTranslations(Environment* ev, 
												FW_XYSelector direction, 
												FW_Fixed translation,
												FW_Fixed scrollingArea);
#ifdef FW_DEBUG
	virtual void			PrivCheckFrame(Environment* ev) const;
#endif	

private:
	void					PrivAttachScrollBars(Environment* ev);
	
//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	FW_CScrollBar*			fScrollbars[2];
	FW_Boolean				fLiveScrolling;
};

//========================================================================================
// Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CScrollBarScroller::FW_CScrollBarScroller
//----------------------------------------------------------------------------------------

inline FW_CScrollBar* FW_CScrollBarScroller::GetScrollBar(Environment*, FW_XYSelector direction) const
{
	return fScrollbars[direction];
}

//----------------------------------------------------------------------------------------
// FW_CScrollBarScroller::SetAutoScrollInset
//----------------------------------------------------------------------------------------

inline void FW_CScroller::SetAutoScrollInset(Environment*, FW_Fixed inset)
{
	fAutoScrollInset = inset;
}

//----------------------------------------------------------------------------------------
// FW_CScroller::GetScrollUnits
//----------------------------------------------------------------------------------------

inline FW_CPoint FW_CScroller::GetScrollUnits(Environment*) const
{
	return fScrollUnit;
}

//----------------------------------------------------------------------------------------
// FW_CScroller::GetScrollUnit
//----------------------------------------------------------------------------------------

inline FW_Fixed FW_CScroller::GetScrollUnit(Environment*, FW_XYSelector direction) const
{
	return fScrollUnit[direction];
}

//----------------------------------------------------------------------------------------
// FW_CScroller::SetScrollUnits
//----------------------------------------------------------------------------------------

inline void FW_CScroller::SetScrollUnits(Environment*, const FW_CPoint& scrollUnits)
{
	fScrollUnit = scrollUnits;
}

//----------------------------------------------------------------------------------------
// FW_CPrivBaseScroller::IsHandlingPageNavigationKeys
//----------------------------------------------------------------------------------------

inline FW_Boolean FW_CPrivBaseScroller::IsHandlingPageNavigationKeys(Environment*) const
{
	return fHandlePageNavigationKeys;
}

//----------------------------------------------------------------------------------------
// FW_CScroller::ChangeHandlingOfPageNavigationKeys
//----------------------------------------------------------------------------------------

inline void FW_CScroller::ChangeHandlingOfPageNavigationKeys(Environment*, FW_Boolean state)
{
	fHandlePageNavigationKeys = state;
}

#endif
